#ifndef LINEINTERPOLATOR2D_H
#define LINEINTERPOLATOR2D_H

class LineInterpolator2D;

class CurrentXSmallerCurrentYSmaller {
public:
  bool operator()(const LineInterpolator2D &, const LineInterpolator2D &) const;
  bool operator()(const LineInterpolator2D *, const LineInterpolator2D *) const;
};

class LineInterpolator2D {
  // Objects are initialized with a line between (x1, y1), (x2, y2).
  // Iteration always begins at (x1, y1) and operator++() walks a step on the
  // line, increasing the x-coordinate by 1.
  //
  // If we call y' the floating point y component of the current
  // position, then GetY() returns floor(y').
  // 
  // If IsYExact() is true, the point (GetX(), GetY()) is exactly on the line.
  //
  // Only integer arithmetic is used.

public:
  LineInterpolator2D(int x1, int y1, int x2, int y2);

  void GoToBegin();
  LineInterpolator2D& operator++(); 
  int GetX() const;
  int GetY() const;
  bool IsYExact() const;

  // Past the end check
  bool IsAtEnd() const;

  // Exactly at the last valid position check
  bool IsAtLastElement() const;

protected:
  int m_x1;
  int m_x2;
  int m_y1;
  int m_y2;
  int m_x;
  int m_y;
  int m_xinc;
  int m_yinc;
  int m_yerror;
  int m_yerrorinc;
  int m_ymaxerror;
  int m_yoverflow;
  int m_yreduceerror;
  int m_stepsleft;

private:
  int ErrorTerm() const;

  friend class CurrentXSmallerCurrentYSmaller;
};
#endif
