#include "LineInterpolator2D.h"
#include <iostream>

void test(int x1, int y1, int x2, int y2) {
  LineInterpolator2D l(x1, y1, x2, y2);

  // Test for correct initialization
  if (l.GetX() != x1) {
    std::cerr << "ERROR: Line doesn't start at x1=" << x1 << "\n";
  }
  if (l.GetY() != y1) {
    std::cerr << "ERROR: Line doesn't start at y1=" << y1 << "\n";
  }
  if (!l.IsYExact()) {
    std::cerr << "ERROR: First y-Position is not exact!\n";
  }

  // Step through the line and test the coordinates
  int dx = x2-x1;
  if (dx != 0) {
    int dy = y2-y1;
    for (int i = 0; i <= std::abs(dx); ++i) {
      int x = l.GetX();
      int y = l.GetY();
      int validx = x1+i*dx/std::abs(dx); 
      int validy = int(floor(y1 + double(i)/double(std::abs(dx))*double(dy)));

      if (x != validx)
        std::cerr << "ERROR: current x should be " << validx << " but is " << x << "\n";
      if (y != validy)
        std::cerr << "ERROR: current y should be " << validy << " but is " << y << "\n";
      ++l;
    }

    // Test end conditions
    if (!l.IsAtEnd()) {
      std::cerr << "ERROR: LineInterpolator2D should have reached it's end!\n";
    }
  }
}

int main(int argc, char *argv[]) {
  // Test empty and straight lines
  test(0, 0, 0, 0);

  test(0, 0, 0, 10);
  test(0, 0, 10, 0);
  test(0, 10, 0, 0);
  test(10, 0, 0, 0);

  // Test diagonal lines
  test(0, 0, 10, 10);
  test(10, 10, 0, 0);
  test(10, 0, 0, 10);
  test(0, 10, 10, 0);

  // Test lines with some exact inner points
  test(0, 0, 8, 12);
  test(8, 0, 0, 12);
  test(0, 12, 8, 0);
  test(8, 12, 0, 0);

  test(0, 0, 12, 8);
  test(12, 0, 0, 8);
  test(0, 8, 12, 0);
  test(12, 8, 0, 0);

  // Test lines without exact inner points
  test(0, 0, 7, 11);
  test(7, 0, 0, 11);
  test(0, 11, 7, 0);
  test(7, 11, 0, 0);

  test(0, 0, 11, 7);
  test(11, 0, 0, 7);
  test(0, 7, 11, 0);
  test(11, 7, 0, 0);

  return 0;
}
