#include "PolygonScanConverter.h"
#include <iostream>
#include "itkImage.h"
#include "itkImageFileWriter.h"
#include "itkRegion.h"
#include "itkTimeProbe.h"

void test(PolygonScanConverter::VertexListType &vertices, char *filename) {
  const int Dimension = 2;
  typedef unsigned char PixelType;
  typedef itk::Image<PixelType, Dimension> ImageType;
  typedef ImageType::RegionType::IndexType IndexType;
  typedef IndexType::IndexValueType IndexValueType;
  
  // Set up a test image
  ImageType::RegionType::IndexType index;
  index.Fill(0);

  ImageType::RegionType::SizeType size;
  size.Fill(800);

  ImageType::RegionType region;
  region.SetIndex(index);
  region.SetSize(size);

  ImageType::Pointer output = ImageType::New();
  output->SetRegions(region);
  output->Allocate();
  output->FillBuffer(0);


  // Draw the polygon
  PolygonScanConverter it(vertices);
  it.GoToBegin();
  while (!it.IsAtEnd()) {
    index[0] = it.GetX();
    index[1] = it.GetY();
    output->SetPixel(index, 100);
    ++it;
  }
  
  // And write a file
  typedef itk::ImageFileWriter<ImageType> WriterType;
  WriterType::Pointer writer = WriterType::New();
  writer->SetInput(output);
  writer->SetFileName(filename);
  writer->Update();
}

void speedtest(PolygonScanConverter::VertexListType &vertices) {
  // Draw the polygon
  PolygonScanConverter it(vertices);
  it.GoToBegin();
  while (!it.IsAtEnd()) {
    ++it;
  }
}

int main(int argc, char *argv[]) {
  PolygonScanConverter::VertexListType vertices;
  PolygonScanConverter::VertexType v;

  // scan convert a cube
  vertices.clear();
  v.x = 1; v.y = 1;
  vertices.push_back(v);
  v.x = 78; v.y = 1;
  vertices.push_back(v);
  v.x = 78; v.y = 78;
  vertices.push_back(v);
  v.x = 1; v.y = 78;
  vertices.push_back(v);
  test(vertices, "cube.png");

  // scan convert a diamond
  vertices.clear();
  v.x = 40; v.y = 10;
  vertices.push_back(v);
  v.x = 70; v.y = 40;
  vertices.push_back(v);
  v.x = 40; v.y = 70;
  vertices.push_back(v);
  v.x = 10; v.y = 40;
  vertices.push_back(v);
  test(vertices, "diamond.png");

  // scan convert a bow tie
  vertices.clear();
  v.x = 10; v.y = 10;
  vertices.push_back(v);
  v.x = 69; v.y = 69;
  vertices.push_back(v);
  v.x = 69; v.y = 10;
  vertices.push_back(v);
  v.x = 10; v.y = 69;
  vertices.push_back(v);
  test(vertices, "bowtie.png");


  // scan convert a random figure
  vertices.clear();
  for (int i = 0; i < 50; ++i) {
    v.x = rand() % 800;
    v.y = rand() % 800;
    // std::cerr << " (" << v.x << "," << v.y << ")";
    vertices.push_back(v);
  }
  test(vertices, "random.png");

  itk::TimeProbe t;
  int numPoints;
  if (argc == 1)
    numPoints = 50;
  else
    numPoints = atoi(argv[1]);

  for (int j = 0; j < 10; ++j) {
    vertices.clear();
    for (int i = 0; i < numPoints; ++i) {
      v.x = rand() % 800-400;
      v.y = rand() % 800-400;
      vertices.push_back(v);
    }
    t.Start();
    speedtest(vertices);
    t.Stop();
  }
  std::clog << "Number of points in poly\tMean time to iterate\n";
  std::clog << numPoints << "\t" << t.GetMeanTime() << "\n";
  return 0;
}
